////////////////////////////////////////////////////////////////////////
//
//        File  Name : server.cc
//        Class Name : server
//        Author     : Young-ki Hwang
//        Date       : April 30, 1996
//        Description: This is methods of "server" class. 
//                     
//
///////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include "server.h"

// From the C library functions
extern "C" bzero(char *, int);
extern "C" bind(int, struct sockaddr *, int);
extern "C" listen(int, int);
extern "C" accept(int, struct sockaddr *, int *);
extern "C" close(int);



//
//  constructor: server()
//  Set up the server machine IP address.
//
server::server(char *str)
{
   address = str;
}



//
//  initialize()
//  Server register its well-known address with the system. It tells the
//  system "this is my address and any message received for this address
//  are to be given to me"
//
server::initialize()
{
   // initialize addresses.
   bzero((char *) &serv_addr, sizeof(serv_addr));      // Initialize      
   serv_addr.sin_family      = AF_INET;                // Internet family 
   serv_addr.sin_port        = htons(SERVER_PORT);     // Port number    
   serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);      // Hostid  

   // assign a name to an unnamed socket.
   if ( bind(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0 )
     { cout << "server: can't bind local address.";
        exit(1);
     }
}


//
//  ready_connection()
//  Make the server to be reday for receive the message.
//
server::ready_connection()
{
   // waiting for any connection.
   listen(sockfd, SOMAXCONN);

   // make a socket file descriptor to be non-blocking for I/O.
   if(fcntl(sockfd, F_SETFL, FNDELAY) < 0)
    { cout<<"Error : fcntl call ";
      exit(1);
     }
}



//
//  make_connection()
//  Accept a connection from the client. If it succeed return 1,
//  otherwise return 0. It opens new socket file descriptor.
//
int server::make_connection()
{   int clilen;
    
    clilen = sizeof(client_addr);

    // accept a connection which is pending.
    newsockfd = accept(sockfd, (struct sockaddr *) &client_addr, &clilen);

    if ( newsockfd < 0 )
     {  if(errno == EWOULDBLOCK) return(0);
       else
         { cout<< "server: can't accept a connection.\n";
	   exit(1);
	 }
      }
    return(1);
}


//
//  get_newsockfd()
//  Get new socket file descriptor. 
//
int server::get_newsockfd()
{
   return(newsockfd);
}


//
// close_newsocket()
// Close new socket file descriptor.
//
server::close_newsocket()
{
    close(newsockfd);
}


//
//  close_oldsocket()
//  close old socket file descritor
//
server::close_oldsocket()
{
    close_socket();
}



